import random

###################################
# Ao repetir a operação de somar  #
# o quadrado dos digitos de um    #
# numero, eventualmente obtemos   #
# 1 ou 89                         #
# Verifique quantos chegam em 89  #
###################################
def ultimo_digito(n):
  return n%10

def remove_ultimo(n):
  return n//10

def soma_quadrados(n):
  soma = 0
  while n != 0:    
    soma = soma + ultimo_digito(n)**2
    n = remove_ultimo(n)
  return soma

def gera_cadeia(n):
  while n != 1 and n != 89:
    n = soma_quadrados(n)    
  return n

def conta_numeros(n):
  conta = 0
  for i in range(1,n):
    num_final = gera_cadeia(i)
    if num_final == 89:
      conta = conta + 1
  return conta

###################################
# Estimando o valor de pi através #
# do método de Monte Carlo        #
###################################
def acertou(x, y):
  return (x - 0.5)**2 + (y - 0.5)**2 <= 0.25

def monte_carlo(n):
  no_alvo = 0
  
  for i in range(1, n+1):
    x = random.random()
    y = random.random()
    if acertou(x, y):
      no_alvo = no_alvo + 1

  meu_pi = 4*no_alvo/n
  
  return meu_pi

###################################
# Um par de números a, b são      #
# amigáveis se a != b e           #
# soma_div(a)==b e soma_div(b)==a #
# Verifique a soma dos n primeiros#
# numeros amigaveis               #
###################################
def divisivel(x, n):
  return x%n==0

def soma_divisores(n):
  soma = 0
  for i in range(1,n):
    if divisivel(n, i):
      soma = soma + i
  return soma

def amigavel(a, b):
  return a != b and soma_divisores(a)==b and soma_divisores(b) == a

def soma_amigaveis(n):
  soma = 0
  conta = 0
  a = 1
  while conta != n:
    b = soma_divisores(a)
    if a<b and soma_divisores(b)==a:
      soma = soma + a + b
    a = a + 1
  return soma

###################################
# Verificador de Luhn             #
#                                 #
# algoritmo para verificar numero #
# de cartão de crédito.           #
# da direita para a esquerda      #
# dobre o valor a cada segundo    #
# digito. Se maior que 9, some    #
# Some todos os digitos           #
# Se for divisivel por 10         #
#  o numero é valido              #
#                                 #
###################################
def eh_par(x):
  return divisivel(x,2)

def insere_digito(x, d):
  if x==0:
    return 10*d
  else:
    mult = 1
    y = 0

    while x != 0:
      y = y + mult*ultimo_digito(x)
      mult = mult*10
      x = remove_ultimo(x)

    return y + mult*d

def soma_digitos(n):
  soma = 0
  while n != 0:    
    soma = soma + ultimo_digito(n)
    n = remove_ultimo(n)
  return soma

def dobra_digitos_pares(x):
  y = 0
  i = 1
  while x!=0:
    digito = ultimo_digito(x)
    x = remove_ultimo(x)
    if eh_par(i):
      digito = soma_digitos(2*digito)
    y = insere_digito(y, digito)
    i = i + 1
  return y

def luhn(x):
  x = dobra_digitos_pares(x)
  if divisivel(soma_digitos(x), 10):
    return True
  else:
    return False
